@@ -4,7 +4,7 @@ import re
 import traceback
 import typing
 from collections import defaultdict
-from typing import Optional
+from typing import Any, Callable, Coroutine, Literal, Optional, Type, Union, overload
 
 import nextcord
 import nextcord.http
@@ -18,7 +18,15 @@ from .commands import (
     TopLevelCommand,
 )
 
+if typing.TYPE_CHECKING:
+    from .groups import CommandGroup
+
 logger = logging.getLogger("dslash")
+AsyncFunc = Callable[..., Coroutine]
+AsyncWrapper = Callable[[AsyncFunc], AsyncFunc]
+
+GUILD_ID_DEFAULT = "default"
+GuildID = Union[None, int, Literal["default"]]
 
 
 class CommandClient(nextcord.Client):
@@ -47,6 +55,64 @@ class CommandClient(nextcord.Client):
         self._commands = defaultdict(dict)
         self._commands_by_id: dict[int, TopLevelCommand] = {}
         self._http: nextcord.http.HTTPClient = self._connection.http
+        self._event_handlers: dict[str, list[AsyncFunc]] = defaultdict(list)
+
+    def dispatch(self, event_name: str, *args: Any, **kwargs: Any) -> None:
+        """Dispatch an event and call additional handlers."""
+        super().dispatch(event_name, *args, **kwargs)
+        for handler in self._event_handlers[event_name]:
+            self._schedule_event(handler, event_name, *args, **kwargs)
+
+    @overload
+    def listener(self, value: AsyncFunc) -> AsyncFunc:
+        """Register an event listener for the event with the function's name.
+
+        The function should be named "on_" followed by the event name, for example:
+
+        ```python
+        @client.listener
+        async def on_message(message):
+            ...
+        ```
+
+        This would register a handler for the "message" event.
+        """
+        ...
+
+    @overload
+    def listener(self, value: str) -> AsyncWrapper:
+        """Register an event listener with the given name.
+
+        For example:
+
+        ```python
+        @client.listener("message")
+        async def my_message_handler(message):
+            ...
+        ```
+        """
+        ...
+
+    def listener(self, value: Union[str, AsyncFunc]) -> Union[AsyncFunc, AsyncWrapper]:
+        """Register an event listener."""
+        if isinstance(value, str):
+
+            def wrapper(handler: AsyncFunc) -> AsyncFunc:
+                self.add_listener(value, handler)
+                return handler
+
+            return wrapper
+        elif callable(value):
+            self.add_listener(value.__name__.removeprefix("on_"), value)
+            return value
+        else:
+            raise TypeError(
+                "CommandClient.listener should be passed an event name or event handler."
+            )
+
+    def add_listener(self, event_name: str, handler: AsyncFunc):
+        """Register a function as an event listener."""
+        self._event_handlers[event_name].append(handler)
 
     async def on_interaction(self, interaction: nextcord.Interaction):
         """Handle a slash command interaction being sent.
@@ -159,10 +225,14 @@ class CommandClient(nextcord.Client):
             error.args = (error_string, *error.args[1:])
         raise error
 
+    def _store_command(self, command: TopLevelCommand):
+        """Store a command to be registered on login."""
+        self._commands[command.guild_id][command.name] = command
+
     def command(
         self,
         *,
-        guild_id: Optional[int] = None,
+        guild_id: GuildID = GUILD_ID_DEFAULT,
         default_permission: bool = True,
         name: Optional[str] = None,
         description: Optional[str] = None,
@@ -229,75 +299,49 @@ class CommandClient(nextcord.Client):
             client=self,
             name=name,
             description=description,
-            guild_id=guild_id or self.guild_id,
+            guild_id=self.guild_id if guild_id == GUILD_ID_DEFAULT else guild_id,
             default_permission=default_permission,
             permissions=permissions,
         )
 
-    def group(
-        self,
-        name: str,
-        description: str,
-        *,
-        guild_id: Optional[int] = None,
-        default_permission: bool = True,
-        permissions: Permissions = None,
-    ) -> SlashCommandGroup:
-        """Create a new top-level command group.
-
-        - `name` (`str`)
-
-          The name of the command group.
-
-        - `description` (`str`)
-
-          A brief description of the command group.
-
-        - `guild_id` (`Optional[int]`)
-
-          The ID of a guild to limit the command to.
-
-        - `default_permission` (default `True`)
-
-          Whether or not this command should be usable for people where no
-          relevant permissions have been set.
-
-        - `permissions` (`Optional[Union[PermissionsSetter, list[PermissionsSetter]]`)
-
-          Either a single permission or a list of permissions (or `None`, the default).
+    def group(self, group: Union[SlashCommandGroup, Type["CommandGroup"]]) -> SlashCommandGroup:
+        """Register a top-level command group.
 
         Example usage:
 
         ```python
         client = CommandClient(guild_id=GUILD_ID)
-        group = client.group(
-            'users',
-            'Commands to manage user profiles.',
-            default_permission=False,
-            permissions=allow_roles(ADMIN_ROLE_ID)
-        )
-
-        @group.subcommand()
-        async def view(
-                interaction: Interaction,
-                user: User = option('The user to view.')):
-            \"""View a user's profile.\"""
-            ...
 
+        @client.group
         @allow_roles(ADMIN_ROLE_ID)
-        @group.subcommand(default_permission=False)
-        async def clear(interaction: Interaction):
-            \"""Clear all user profiles.\"""
-            ...
+        class Profiles(CommandGroup, default_permission=False):
+            \"""Commands to manage user profiles.\"""
+
+            @subcommand()
+            async def create(self, interaction: Interaction, user: User, age: int):
+                \"""Create a new profile for someone.
+
+                :user: The user to create the profile for.
+                :age: The age of the user, in years.
+                \"""
+                ...
+
+            @subcommand()
+            async def delete(self, interaction: Interaction, user: User):
+                \"""Delete someone's profile.
+
+                :user: The person who's profile you want to delete.
+                \"""
+                ...
 
         client.run(TOKEN)
         ```
         """
-        return SlashCommandGroup(
-            client=self,
-            name=name,
-            description=description,
-            guild_id=guild_id or self.guild_id,
-            default_permission=default_permission,
-            permissions=permissions,
-        )
+        # The metaclass of SlashCommandGroup means that the name type checkers
+        # think points to the type itself will in fact point to an instance of
+        # SlashCommandGroup.
+        group_: SlashCommandGroup = group  # type: ignore
+        if group_.guild_id == GUILD_ID_DEFAULT:
+            group_.guild_id = self.guild_id
+        self._store_command(group_)
+        return group_
